(function(window) {
    var Waves = Waves || {};
    var $$ = document.querySelectorAll.bind(document);

    // Find exact position of element
    function isWindow(obj) {
        return obj !== null && obj === obj.window;
    }

    function getWindow(elem) {
        return isWindow(elem) ? elem : elem.nodeType === 9 && elem.defaultView;
    }

    function offset(elem) {

        var docElem, win,
            box = {
                top: 0,
                left: 0
            },
            doc = elem && elem.ownerDocument;

        docElem = doc.documentElement;

        if (typeof elem.getBoundingClientRect !== typeof undefined) {
            box = elem.getBoundingClientRect();
        }
        win = getWindow(doc);
        return {
            top: box.top + win.pageYOffset - docElem.clientTop,
            left: box.left + win.pageXOffset - docElem.clientLeft
        };
    }

    function convertStyle(obj) {

        var style = '';

        for (var a in obj) {
            if (obj.hasOwnProperty(a)) {
                style += (a + ':' + obj[a] + ';');
            }
        }

        return style;
    }

    var timer = null
    var Effect = {
        duration: 750,
        show: function(e) {
            // Disable right click
            if (e.button === 2) {
                return false;
            }

            var el = this;
            var pageX, pageY;
            if ('touches' in e) {
                pageX = e.touches[0].pageX
                pageY = e.touches[0].pageY
            } else {
                pageX = e.pageX
                pageY = e.pageY
            }

            // Create ripple
            var ripple = document.createElement('div');
            ripple.className = 'waves-ripple';
            el.appendChild(ripple);

            // Get click coordinate and element witdh
            var pos = offset(el);
            var relativeY = (pageY - pos.top);
            var relativeX = (pageX - pos.left);

            var posToLeft = relativeX
            var posToRight = Math.abs(pos.left + el.clientWidth - pageX)
            var halfWidth = Math.max(posToLeft, posToRight)

            var scale = 'scale(' + ((halfWidth * 2 + 20) / 100) + ')';

            // Attach data to element
            ripple.setAttribute('data-hold', Date.now());
            ripple.setAttribute('data-scale', scale);
            ripple.setAttribute('data-x', relativeX);
            ripple.setAttribute('data-y', relativeY);

            timer = setTimeout(function() {
                Effect._show(el)
            }, 300)
        },
        _show: function(el, duration) {
            var ripple = null
            var childrenLength = el.children.length
            var duration = duration || Effect.duration
            for (var a = 0; a < childrenLength; a++) {
                if (el.children[a].className.indexOf('waves-ripple') !== -1) {
                    ripple = el.children[a];
                    continue;
                }
            }

            if (!ripple) {
                return false;
            }

            var scale = ripple.getAttribute('data-scale')
            var relativeY = ripple.getAttribute('data-y')
            var relativeX = ripple.getAttribute('data-x')
            var rippleStyle = {
                'top': relativeY + 'px',
                'left': relativeX + 'px'
            };
            // Scale the ripple
            rippleStyle['-webkit-transform'] = scale;
            rippleStyle['-moz-transform'] = scale;
            rippleStyle['-ms-transform'] = scale;
            rippleStyle['-o-transform'] = scale;
            rippleStyle.transform = scale;
            rippleStyle.opacity = '1';

            rippleStyle['-webkit-transition-duration'] = duration + 'ms';
            rippleStyle['-moz-transition-duration'] = duration + 'ms';
            rippleStyle['-o-transition-duration'] = duration + 'ms';
            rippleStyle['transition-duration'] = duration + 'ms';

            ripple.setAttribute('style', convertStyle(rippleStyle));
            return ripple
        },
        hide: function() {
            var el = this;
            var width = el.clientWidth * 1.4;

            // Get first ripple
            var ripple = null;
            var childrenLength = el.children.length;

            for (var a = 0; a < childrenLength; a++) {
                if (el.children[a].className.indexOf('waves-ripple') !== -1) {
                    ripple = el.children[a];
                    continue;
                }
            }

            if (!ripple) {
                return false;
            }

            var diff = Date.now() - Number(ripple.getAttribute('data-hold'))
            if (diff < 200) {
                clearTimeout(timer)
                Effect.quickInOut(el)
                return
            }

            var relativeX = ripple.getAttribute('data-x');
            var relativeY = ripple.getAttribute('data-y');
            var scale = ripple.getAttribute('data-scale');

            // Get delay beetween mousedown and mouse leave
            var delay = 500 - diff;

            if (delay < 0) {
                delay = 0;
            }

            // Fade out ripple after delay
            setTimeout(function() {
                ripple.style.opacity = 0
                setTimeout(function() {

                    try {
                        el.removeChild(ripple);
                    } catch (e) {
                        return false;
                    }
                }, Effect.duration);

            }, delay);

        },
        quickInOut: function(el) {
            var ripple = Effect._show(el, 360)
            if (ripple) {
                setTimeout(function() {
                    ripple.style.opacity = 0
                    setTimeout(function() {
                        try {
                            el.removeChild(ripple);
                        } catch (e) {
                            return false;
                        }
                    }, 360);
                }, 360)
            }
        },
        cancel: function() {
            // Get first ripple
            var el = this;
            var ripple = null;
            var childrenLength = el.children.length;

            for (var a = 0; a < childrenLength; a++) {
                if (el.children[a].className.indexOf('waves-ripple') !== -1) {
                    ripple = el.children[a];
                    continue;
                }
            }

            if (!ripple) {
                return false;
            }

            el.removeChild(ripple)
        }
    };

    Waves.displayEffect = function(options) {
        options = options || {};

        if ('duration' in options) {
            Effect.duration = options.duration;
        }

        Array.prototype.forEach.call($$('.waves-effect'), function(i) {
            if ('ontouchstart' in window) {
                i.addEventListener('touchstart', Effect.show, false);
                i.addEventListener('touchend', Effect.hide, false);
                i.addEventListener('touchmove', Effect.cancel, false);
                i.addEventListener('touchcancel', Effect.hide, false);
            } else {
                i.addEventListener('mousedown', Effect.show, false);
                i.addEventListener('mouseup', Effect.hide, false);
                i.addEventListener('mouseleave', Effect.hide, false);
            }

        });

    };

    window.Waves = Waves;

})(window);
